@@ -1,8 +1,11 @@
-use crate::api::managed_types::managed_buffer_api_node::{
-    unsafe_buffer_load_address, unsafe_buffer_load_token_identifier,
+use crate::{
+    api::managed_types::managed_buffer_api_node::{
+        unsafe_buffer_load_address, unsafe_buffer_load_token_identifier,
+    },
+    VmApiImpl,
 };
 use elrond_wasm::{
-    api::BlockchainApi,
+    api::{BlockchainApi, BlockchainApiImpl, Handle, ManagedTypeApi},
     types::{
         Address, BigUint, Box, EsdtTokenData, EsdtTokenType, ManagedAddress, ManagedBuffer,
         ManagedType, ManagedVec, TokenIdentifier, H256,
@@ -134,7 +137,16 @@ extern "C" {
     );
 }
 
-impl BlockchainApi for crate::VmApiImpl {
+impl BlockchainApi for VmApiImpl {
+    type BlockchainApiImpl = VmApiImpl;
+
+    #[inline]
+    fn blockchain_api_impl() -> Self::BlockchainApiImpl {
+        VmApiImpl {}
+    }
+}
+
+impl BlockchainApiImpl for VmApiImpl {
     #[inline]
     fn get_sc_address_legacy(&self) -> Address {
         unsafe {
@@ -146,11 +158,11 @@ impl BlockchainApi for crate::VmApiImpl {
 
     #[inline]
     #[cfg(not(feature = "unmanaged-ei"))]
-    fn get_sc_address(&self) -> ManagedAddress<Self> {
+    fn get_sc_address_handle(&self) -> Handle {
         unsafe {
             let handle = mBufferNew();
             managedSCAddress(handle);
-            ManagedAddress::from_raw_handle(handle)
+            handle
         }
     }
 
@@ -165,11 +177,11 @@ impl BlockchainApi for crate::VmApiImpl {
 
     #[inline]
     #[cfg(not(feature = "unmanaged-ei"))]
-    fn get_owner_address(&self) -> ManagedAddress<Self> {
+    fn get_owner_address_handle(&self) -> Handle {
         unsafe {
             let handle = mBufferNew();
             managedOwnerAddress(handle);
-            ManagedAddress::from_raw_handle(handle)
+            handle
         }
     }
 
@@ -179,8 +191,8 @@ impl BlockchainApi for crate::VmApiImpl {
     }
 
     #[inline]
-    fn get_shard_of_address(&self, address: &ManagedAddress<Self>) -> u32 {
-        unsafe { getShardOfAddress(unsafe_buffer_load_address(address)) as u32 }
+    fn get_shard_of_address(&self, address_handle: Handle) -> u32 {
+        unsafe { getShardOfAddress(unsafe_buffer_load_address(address_handle)) as u32 }
     }
 
     #[inline]
@@ -189,8 +201,8 @@ impl BlockchainApi for crate::VmApiImpl {
     }
 
     #[inline]
-    fn is_smart_contract(&self, address: &ManagedAddress<Self>) -> bool {
-        unsafe { isSmartContract(unsafe_buffer_load_address(address)) > 0 }
+    fn is_smart_contract(&self, address_handle: Handle) -> bool {
+        unsafe { isSmartContract(unsafe_buffer_load_address(address_handle)) > 0 }
     }
 
     #[inline]
@@ -204,27 +216,27 @@ impl BlockchainApi for crate::VmApiImpl {
 
     #[inline]
     #[cfg(not(feature = "unmanaged-ei"))]
-    fn get_caller(&self) -> ManagedAddress<Self> {
+    fn get_caller_handle(&self) -> Handle {
         unsafe {
             let handle = mBufferNew();
             managedCaller(handle);
-            ManagedAddress::from_raw_handle(handle)
+            handle
         }
     }
 
-    fn get_balance_legacy(&self, address: &Address) -> BigUint<Self> {
+    fn get_balance_legacy(&self, address: &Address) -> Handle {
         unsafe {
             let balance_handle = bigIntNew(0);
             bigIntGetExternalBalance(address.as_ref().as_ptr(), balance_handle);
-            BigUint::from_raw_handle(balance_handle)
+            balance_handle
         }
     }
 
-    fn get_balance(&self, address: &ManagedAddress<Self>) -> BigUint<Self> {
+    fn get_balance_handle(&self, address_handle: Handle) -> Handle {
         unsafe {
             let balance_handle = bigIntNew(0);
-            bigIntGetExternalBalance(unsafe_buffer_load_address(address), balance_handle);
-            BigUint::from_raw_handle(balance_handle)
+            bigIntGetExternalBalance(unsafe_buffer_load_address(address_handle), balance_handle);
+            balance_handle
         }
     }
 
@@ -239,7 +251,9 @@ impl BlockchainApi for crate::VmApiImpl {
 
     #[inline]
     #[cfg(not(feature = "unmanaged-ei"))]
-    fn get_state_root_hash(&self) -> elrond_wasm::types::ManagedByteArray<Self, 32> {
+    fn get_state_root_hash<M: ManagedTypeApi>(
+        &self,
+    ) -> elrond_wasm::types::ManagedByteArray<M, 32> {
         unsafe {
             let result_handle = mBufferNew();
             managedGetStateRootHash(result_handle);
@@ -258,7 +272,7 @@ impl BlockchainApi for crate::VmApiImpl {
 
     #[inline]
     #[cfg(not(feature = "unmanaged-ei"))]
-    fn get_tx_hash(&self) -> elrond_wasm::types::ManagedByteArray<Self, 32> {
+    fn get_tx_hash<M: ManagedTypeApi>(&self) -> elrond_wasm::types::ManagedByteArray<M, 32> {
         unsafe {
             let result_handle = mBufferNew();
             managedGetOriginalTxHash(result_handle);
@@ -302,7 +316,9 @@ impl BlockchainApi for crate::VmApiImpl {
 
     #[inline]
     #[cfg(not(feature = "unmanaged-ei"))]
-    fn get_block_random_seed(&self) -> elrond_wasm::types::ManagedByteArray<Self, 48> {
+    fn get_block_random_seed<M: ManagedTypeApi>(
+        &self,
+    ) -> elrond_wasm::types::ManagedByteArray<M, 48> {
         unsafe {
             let result_handle = mBufferNew();
             managedGetBlockRandomSeed(result_handle);
@@ -341,7 +357,9 @@ impl BlockchainApi for crate::VmApiImpl {
 
     #[inline]
     #[cfg(not(feature = "unmanaged-ei"))]
-    fn get_prev_block_random_seed(&self) -> elrond_wasm::types::ManagedByteArray<Self, 48> {
+    fn get_prev_block_random_seed<M: ManagedTypeApi>(
+        &self,
+    ) -> elrond_wasm::types::ManagedByteArray<M, 48> {
         unsafe {
             let result_handle = mBufferNew();
             managedGetPrevBlockRandomSeed(result_handle);
@@ -350,31 +368,31 @@ impl BlockchainApi for crate::VmApiImpl {
     }
 
     #[inline]
-    fn get_current_esdt_nft_nonce(
+    fn get_current_esdt_nft_nonce<M: ManagedTypeApi>(
         &self,
-        address: &ManagedAddress<Self>,
-        token: &TokenIdentifier<Self>,
+        address: &ManagedAddress<M>,
+        token: &TokenIdentifier<M>,
     ) -> u64 {
         unsafe {
             getCurrentESDTNFTNonce(
-                unsafe_buffer_load_address(address),
-                unsafe_buffer_load_token_identifier(token),
+                unsafe_buffer_load_address(address.get_raw_handle()),
+                unsafe_buffer_load_token_identifier(token.get_raw_handle()),
                 token.len() as i32,
             ) as u64
         }
     }
 
-    fn get_esdt_balance(
+    fn get_esdt_balance<M: ManagedTypeApi>(
         &self,
-        address: &ManagedAddress<Self>,
-        token: &TokenIdentifier<Self>,
+        address: &ManagedAddress<M>,
+        token: &TokenIdentifier<M>,
         nonce: u64,
-    ) -> BigUint<Self> {
+    ) -> BigUint<M> {
         unsafe {
             let balance_handle = bigIntNew(0);
             bigIntGetESDTExternalBalance(
-                unsafe_buffer_load_address(address),
-                unsafe_buffer_load_token_identifier(token),
+                unsafe_buffer_load_address(address.get_raw_handle()),
+                unsafe_buffer_load_token_identifier(token.get_raw_handle()),
                 token.len() as i32,
                 nonce as i64,
                 balance_handle,
@@ -385,12 +403,12 @@ impl BlockchainApi for crate::VmApiImpl {
     }
 
     #[cfg(feature = "unmanaged-ei")]
-    fn get_esdt_token_data(
+    fn get_esdt_token_data<M: ManagedTypeApi>(
         &self,
-        m_address: &ManagedAddress<Self>,
-        token: &TokenIdentifier<Self>,
+        m_address: &ManagedAddress<M>,
+        token: &TokenIdentifier<M>,
         nonce: u64,
-    ) -> EsdtTokenData<Self> {
+    ) -> EsdtTokenData<M> {
         use elrond_wasm::types::BoxedBytes;
         let address = m_address.to_address();
         unsafe {
@@ -476,12 +494,12 @@ impl BlockchainApi for crate::VmApiImpl {
     }
 
     #[cfg(not(feature = "unmanaged-ei"))]
-    fn get_esdt_token_data(
+    fn get_esdt_token_data<M: ManagedTypeApi>(
         &self,
-        address: &ManagedAddress<Self>,
-        token: &TokenIdentifier<Self>,
+        address: &ManagedAddress<M>,
+        token: &TokenIdentifier<M>,
         nonce: u64,
-    ) -> EsdtTokenData<Self> {
+    ) -> EsdtTokenData<M> {
         let managed_token_id = token.as_managed_buffer();
         unsafe {
             let value_handle = bigIntNew(0);
@@ -533,10 +551,57 @@ impl BlockchainApi for crate::VmApiImpl {
         }
     }
 
-    #[cfg(feature = "vm-esdt-local-roles")]
-    fn get_esdt_local_roles(
+    #[cfg(not(feature = "vm-esdt-local-roles"))]
+    fn get_esdt_local_roles<M: ManagedTypeApi>(
         &self,
-        token_id: &TokenIdentifier<Self>,
+        token_id: &TokenIdentifier<M>,
+    ) -> elrond_wasm::types::EsdtLocalRoleFlags {
+        use elrond_wasm::{
+            api::{ErrorApiImpl, ManagedBufferApi, StorageReadApiImpl},
+            storage::StorageKey,
+            types::{EsdtLocalRole, EsdtLocalRoleFlags},
+        };
+
+        let mut key =
+            StorageKey::new(elrond_wasm::storage::protected_keys::ELROND_ESDT_LOCAL_ROLES_KEY);
+        key.append_managed_buffer(token_id.as_managed_buffer());
+        let value_handle = self.storage_load_managed_buffer_raw(key.get_raw_handle());
+        let value_len = self.mb_len(value_handle);
+        const DATA_MAX_LEN: usize = 300;
+        if value_len > DATA_MAX_LEN {
+            self.signal_error(elrond_wasm::err_msg::STORAGE_VALUE_EXCEEDS_BUFFER);
+        }
+        let mut data_buffer = [0u8; DATA_MAX_LEN];
+        // let _ = value_mb.load_slice(0, );
+        let _ = self.mb_load_slice(value_handle, 0, &mut data_buffer[..value_len]);
+
+        let mut current_index = 0;
+
+        let mut result = EsdtLocalRoleFlags::NONE;
+
+        while current_index < value_len {
+            // first character before each role is a \n, so we skip it
+            current_index += 1;
+
+            // next is the length of the role as string
+            let role_len = data_buffer[current_index];
+            current_index += 1;
+
+            // next is role's ASCII string representation
+            let end_index = current_index + role_len as usize;
+            let role_name = &data_buffer[current_index..end_index];
+            current_index = end_index;
+
+            result |= EsdtLocalRole::from(role_name).to_flag();
+        }
+
+        result
+    }
+
+    #[cfg(feature = "vm-esdt-local-roles")]
+    fn get_esdt_local_roles<M: ManagedTypeApi>(
+        &self,
+        token_id: &TokenIdentifier<M>,
     ) -> elrond_wasm::types::EsdtLocalRoleFlags {
         let managed_token_id = token_id.as_managed_buffer();
         unsafe {
